# -*- coding: utf-8 -*-

from Products.CMFCore.WorkflowTool import addWorkflowFactory
from Products.DCWorkflow.DCWorkflow import DCWorkflowDefinition

def setupPgforum_workflow(wf):
    "..."
    wf.setProperties(title='PGForum Workflow [PloneGossip]')

    for s in ['archived', 'open', 'closed', 'private', 'public']:
        wf.states.addState(s)
    for t in ['close', 'make_private', 'reopen', 'make_public', 'archive']:
        wf.transitions.addTransition(t)
    for v in ['action', 'time', 'comments', 'actor', 'review_history']:
        wf.variables.addVariable(v)
    for l in []:
        wf.worklists.addWorklist(l)
    for p in ('Access contents information', 'Modify portal content', 'View', 'List folder contents', 'PloneGossip: Add thread', 'PloneGossip: Add post', 'PloneGossip: Review thread', 'PloneGossip: Review post', 'Delete objects'):
        wf.addManagedPermission(p)
        

    ## Initial State
    wf.states.setInitialState('open')

    ## States initialization
    sdef = wf.states['archived']
    sdef.setProperties(title="""Archived""",
                       transitions=('close', 'make_private', 'make_public', 'reopen'))
    sdef.setPermission('Access contents information', 0, ['Manager', 'Owner'])
    sdef.setPermission('Modify portal content', 0, ['Manager'])
    sdef.setPermission('View', 0, ['Manager', 'Owner'])
    sdef.setPermission('List folder contents', 0, ['Manager', 'Owner'])
    sdef.setPermission('PloneGossip: Add thread', 0, ['Manager'])
    sdef.setPermission('PloneGossip: Add post', 0, ['Manager'])
    sdef.setPermission('PloneGossip: Review thread', 0, ['Manager'])
    sdef.setPermission('PloneGossip: Review post', 0, ['Manager'])
    sdef.setPermission('Delete objects', 1, [])

    sdef = wf.states['open']
    sdef.setProperties(title="""Open""",
                       transitions=('archive', 'close', 'make_private', 'make_public'))
    sdef.setPermission('Access contents information', 0, ['Anonymous', 'Manager', 'Member', 'Owner', 'Reviewer'])
    sdef.setPermission('Modify portal content', 0, ['Manager', 'Owner', 'Reviewer'])
    sdef.setPermission('View', 0, ['Anonymous', 'Manager', 'Member', 'Owner', 'Reviewer'])
    sdef.setPermission('List folder contents', 0, ['Anonymous', 'Manager', 'Member', 'Owner', 'Reviewer'])
    sdef.setPermission('PloneGossip: Add thread', 0, ['Manager', 'Member'])
    sdef.setPermission('PloneGossip: Add post', 0, ['Manager', 'Member'])
    sdef.setPermission('PloneGossip: Review thread', 0, ['Manager', 'Reviewer'])
    sdef.setPermission('PloneGossip: Review post', 0, ['Manager', 'Reviewer'])
    sdef.setPermission('Delete objects', 0, ['Manager', 'Reviewer'])

    sdef = wf.states['closed']
    sdef.setProperties(title="""Closed""",
                       transitions=('archive', 'make_public', 'reopen'))
    sdef.setPermission('Access contents information', 0, ['Manager', 'Member', 'Reviewer'])
    sdef.setPermission('Modify portal content', 0, ['Manager'])
    sdef.setPermission('View', 0, ['Manager', 'Member', 'Reviewer'])
    sdef.setPermission('List folder contents', 0, ['Manager', 'Member', 'Reviewer'])
    sdef.setPermission('PloneGossip: Add thread', 0, ['Manager'])
    sdef.setPermission('PloneGossip: Add post', 0, ['Manager'])
    sdef.setPermission('PloneGossip: Review thread', 0, ['Manager'])
    sdef.setPermission('PloneGossip: Review post', 0, ['Manager'])
    sdef.setPermission('Delete objects', 0, ['Manager', 'Reviewer'])

    sdef = wf.states['private']
    sdef.setProperties(title="""Private""",
                       transitions=('archive', 'close', 'make_public', 'reopen'))
    sdef.setPermission('Access contents information', 0, ['Manager', 'Member', 'Owner', 'Reviewer'])
    sdef.setPermission('Modify portal content', 0, ['Manager', 'Owner', 'Reviewer'])
    sdef.setPermission('View', 0, ['Manager', 'Member', 'Owner', 'Reviewer'])
    sdef.setPermission('List folder contents', 0, ['Manager', 'Member', 'Owner', 'Reviewer'])
    sdef.setPermission('PloneGossip: Add thread', 0, ['Manager', 'Member', 'Reviewer'])
    sdef.setPermission('PloneGossip: Add post', 0, ['Manager', 'Member', 'Reviewer'])
    sdef.setPermission('PloneGossip: Review thread', 0, ['Manager', 'Reviewer'])
    sdef.setPermission('PloneGossip: Review post', 0, ['Manager', 'Reviewer'])
    sdef.setPermission('Delete objects', 0, ['Manager', 'Reviewer'])

    sdef = wf.states['public']
    sdef.setProperties(title="""Public""",
                       transitions=('archive', 'close', 'make_private'))
    sdef.setPermission('Access contents information', 1, ['Anonymous', 'Manager', 'Member', 'Owner'])
    sdef.setPermission('Modify portal content', 1, ['Manager', 'Owner'])
    sdef.setPermission('View', 1, ['Anonymous', 'Manager', 'Member', 'Owner'])
    sdef.setPermission('List folder contents', 1, ['Anonymous', 'Manager', 'Member', 'Owner'])
    sdef.setPermission('PloneGossip: Add thread', 1, ['Anonymous', 'Manager', 'Member', 'Owner'])
    sdef.setPermission('PloneGossip: Add post', 1, ['Anonymous', 'Manager', 'Member', 'Owner'])
    sdef.setPermission('PloneGossip: Review thread', 1, ['Manager', 'Owner'])
    sdef.setPermission('PloneGossip: Review post', 1, ['Manager', 'Owner'])
    sdef.setPermission('Delete objects', 0, ['Manager', 'Reviewer'])


    ## Transitions initialization
    tdef = wf.transitions['close']
    tdef.setProperties(title="""Close a forum to post""",
                       new_state_id="""closed""",
                       trigger_type=1,
                       script_name="""""",
                       after_script_name="""""",
                       actbox_name="""Close""",
                       actbox_url="""%(content_url)s/content_close_form""",
                       actbox_category="""workflow""",
                       props={'guard_roles': 'Owner'},
                       )

    tdef = wf.transitions['make_private']
    tdef.setProperties(title="""Make private""",
                       new_state_id="""private""",
                       trigger_type=1,
                       script_name="""""",
                       after_script_name="""""",
                       actbox_name="""Make private""",
                       actbox_url="""%(content_url)s/content_makeprivate_form""",
                       actbox_category="""workflow""",
                       props={'guard_roles': 'Owner'},
                       )

    tdef = wf.transitions['reopen']
    tdef.setProperties(title="""Reopen the forum""",
                       new_state_id="""open""",
                       trigger_type=1,
                       script_name="""""",
                       after_script_name="""""",
                       actbox_name="""Reopen""",
                       actbox_url="""%(content_url)s/content_reopen_form""",
                       actbox_category="""workflow""",
                       props={'guard_roles': 'Owner'},
                       )

    tdef = wf.transitions['make_public']
    tdef.setProperties(title="""Make public""",
                       new_state_id="""public""",
                       trigger_type=1,
                       script_name="""""",
                       after_script_name="""""",
                       actbox_name="""Make public""",
                       actbox_url="""%(content_url)s/content_public_form""",
                       actbox_category="""workflow""",
                       props={'guard_roles': 'Owner; Manager'},
                       )

    tdef = wf.transitions['archive']
    tdef.setProperties(title="""Archive""",
                       new_state_id="""archived""",
                       trigger_type=1,
                       script_name="""""",
                       after_script_name="""""",
                       actbox_name="""Archive""",
                       actbox_url="""%(content_url)s/content_archive_form""",
                       actbox_category="""workflow""",
                       props={'guard_roles': 'Owner'},
                       )

    ## State Variable
    wf.variables.setStateVar('review_state')

    ## Variables initialization
    vdef = wf.variables['action']
    vdef.setProperties(description="""The last transition""",
                       default_value="""""",
                       default_expr="""transition/getId|nothing""",
                       for_catalog=0,
                       for_status=1,
                       update_always=1,
                       props=None)

    vdef = wf.variables['time']
    vdef.setProperties(description="""Time of the last transition""",
                       default_value="""""",
                       default_expr="""state_change/getDateTime""",
                       for_catalog=0,
                       for_status=1,
                       update_always=1,
                       props=None)

    vdef = wf.variables['comments']
    vdef.setProperties(description="""Comments about the last transition""",
                       default_value="""""",
                       default_expr="""python:state_change.kwargs.get('comment', '')""",
                       for_catalog=0,
                       for_status=1,
                       update_always=1,
                       props=None)

    vdef = wf.variables['actor']
    vdef.setProperties(description="""The ID of the user who performed the last transition""",
                       default_value="""""",
                       default_expr="""user/getId""",
                       for_catalog=0,
                       for_status=1,
                       update_always=1,
                       props=None)

    vdef = wf.variables['review_history']
    vdef.setProperties(description="""Provides access to workflow history""",
                       default_value="""""",
                       default_expr="""state_change/getHistory""",
                       for_catalog=0,
                       for_status=0,
                       update_always=0,
                       props={'guard_permissions': 'Request review; Review portal content'})

    ## Worklists Initialization

def createPgforum_workflow(id):
    "..."
    ob = DCWorkflowDefinition(id)
    setupPgforum_workflow(ob)
    return ob

addWorkflowFactory(createPgforum_workflow,
                   id='pgforum_workflow',
                   title='PGForum Workflow [PloneGossip]')
