# -*- coding: utf-8 -*-

from Products.CMFCore.WorkflowTool import addWorkflowFactory
from Products.DCWorkflow.DCWorkflow import DCWorkflowDefinition

def setupPgthread_workflow(wf):
    "..."
    wf.setProperties(title='PGThread Workflow [PloneGossip]')

    for s in ['draft', 'pending', 'closed', 'published']:
        wf.states.addState(s)
    for t in ['reopen', 'submit', 'auto-submit', 'auto-publish', 'publish', 'reject', 'close']:
        wf.transitions.addTransition(t)
    for v in ['action', 'time', 'comments', 'actor', 'review_history']:
        wf.variables.addVariable(v)
    for l in ['reviewer_queue']:
        wf.worklists.addWorklist(l)
    for p in ('Access contents information', 'Modify portal content', 'View', 'PloneGossip: Add post', 'PloneGossip: Review post', 'PloneGossip: Review thread', 'PloneGossip: Add thread'):
        wf.addManagedPermission(p)
        

    ## Initial State
    wf.states.setInitialState('draft')

    ## States initialization
    sdef = wf.states['draft']
    sdef.setProperties(title="""Private draft""",
                       transitions=('auto-submit', 'submit'))
    sdef.setPermission('Access contents information', 0, ['Manager', 'Owner', 'Reviewer'])
    sdef.setPermission('Modify portal content', 0, ['Manager', 'Owner', 'Reviewer'])
    sdef.setPermission('View', 0, ['Manager', 'Owner', 'Reviewer'])
    sdef.setPermission('PloneGossip: Add post', 0, ['Manager'])
    sdef.setPermission('PloneGossip: Review post', 0, ['Manager', 'Reviewer'])
    sdef.setPermission('PloneGossip: Review thread', 0, ['Manager', 'Reviewer'])
    sdef.setPermission('PloneGossip: Add thread', 0, ['Manager'])

    sdef = wf.states['pending']
    sdef.setProperties(title="""Pending""",
                       transitions=('auto-publish', 'publish'))
    sdef.setPermission('Access contents information', 0, ['Manager', 'Owner', 'Reviewer'])
    sdef.setPermission('Modify portal content', 0, ['Manager', 'Reviewer'])
    sdef.setPermission('View', 0, ['Manager', 'Owner', 'Reviewer'])
    sdef.setPermission('PloneGossip: Add post', 0, [])
    sdef.setPermission('PloneGossip: Review post', 0, ['Manager', 'Reviewer'])
    sdef.setPermission('PloneGossip: Review thread', 0, ['Manager', 'Reviewer'])
    sdef.setPermission('PloneGossip: Add thread', 0, [])

    sdef = wf.states['closed']
    sdef.setProperties(title="""Closed""",
                       transitions=('reopen',))
    sdef.setPermission('Access contents information', 1, ['Manager'])
    sdef.setPermission('Modify portal content', 0, ['Manager', 'Reviewer'])
    sdef.setPermission('View', 1, ['Manager'])
    sdef.setPermission('PloneGossip: Add post', 0, [])
    sdef.setPermission('PloneGossip: Review post', 0, ['Manager', 'Reviewer'])
    sdef.setPermission('PloneGossip: Review thread', 0, ['Manager', 'Reviewer'])
    sdef.setPermission('PloneGossip: Add thread', 0, [])

    sdef = wf.states['published']
    sdef.setProperties(title="""Published""",
                       transitions=('close', 'reject'))
    sdef.setPermission('Access contents information', 1, ['Manager'])
    sdef.setPermission('Modify portal content', 0, ['Manager'])
    sdef.setPermission('View', 1, ['Manager'])
    sdef.setPermission('PloneGossip: Add post', 1, ['Manager'])
    sdef.setPermission('PloneGossip: Review post', 1, ['Manager', 'Reviewer'])
    sdef.setPermission('PloneGossip: Review thread', 1, ['Manager', 'Reviewer'])
    sdef.setPermission('PloneGossip: Add thread', 1, ['Manager'])


    ## Transitions initialization
    tdef = wf.transitions['reopen']
    tdef.setProperties(title="""Reopen""",
                       new_state_id="""published""",
                       trigger_type=1,
                       script_name="""""",
                       after_script_name="""""",
                       actbox_name="""Reopen""",
                       actbox_url="""%(content_url)s/content_reopen_form""",
                       actbox_category="""workflow""",
                       props={'guard_roles': 'Owner; Manager'},
                       )

    tdef = wf.transitions['submit']
    tdef.setProperties(title="""Submit""",
                       new_state_id="""pending""",
                       trigger_type=1,
                       script_name="""""",
                       after_script_name="""""",
                       actbox_name="""Submit""",
                       actbox_url="""%(content_url)s/content_submit_form""",
                       actbox_category="""workflow""",
                       props={'guard_roles': 'Owner; Manager'},
                       )

    tdef = wf.transitions['auto-submit']
    tdef.setProperties(title="""Auto submit""",
                       new_state_id="""pending""",
                       trigger_type=0,
                       script_name="""""",
                       after_script_name="""""",
                       actbox_name="""Auto submit""",
                       actbox_url="""%(content_url)s/content_submit_form""",
                       actbox_category="""workflow""",
                       )

    tdef = wf.transitions['auto-publish']
    tdef.setProperties(title="""Auto publish""",
                       new_state_id="""published""",
                       trigger_type=0,
                       script_name="""""",
                       after_script_name="""""",
                       actbox_name="""Auto publish""",
                       actbox_url="""%(content_url)s/content_publish_form""",
                       actbox_category="""workflow""",
                       props={'guard_expr': 'python: not container.getIsModerated() or container.isReviewable()'},
                       )

    tdef = wf.transitions['publish']
    tdef.setProperties(title="""Publish""",
                       new_state_id="""published""",
                       trigger_type=1,
                       script_name="""""",
                       after_script_name="""""",
                       actbox_name="""Publish""",
                       actbox_url="""%(content_url)s/content_publish_form""",
                       actbox_category="""workflow""",
                       props={'guard_permissions': 'PloneGossip: Review post; PloneGossip: Review thread'},
                       )

    tdef = wf.transitions['reject']
    tdef.setProperties(title="""Reject""",
                       new_state_id="""pending""",
                       trigger_type=1,
                       script_name="""""",
                       after_script_name="""""",
                       actbox_name="""Reject""",
                       actbox_url="""%(content_url)s/content_reopen_form""",
                       actbox_category="""workflow""",
                       props={'guard_expr': 'container/getIsModerated', 'guard_permissions': 'PloneGossip: Review post; PloneGossip: Review thread'},
                       )

    tdef = wf.transitions['close']
    tdef.setProperties(title="""Close""",
                       new_state_id="""closed""",
                       trigger_type=1,
                       script_name="""""",
                       after_script_name="""""",
                       actbox_name="""Close""",
                       actbox_url="""%(content_url)s/content_close_form""",
                       actbox_category="""workflow""",
                       props={'guard_roles': 'Owner'},
                       )

    ## State Variable
    wf.variables.setStateVar('review_state')

    ## Variables initialization
    vdef = wf.variables['action']
    vdef.setProperties(description="""The last transition""",
                       default_value="""""",
                       default_expr="""transition/getId|nothing""",
                       for_catalog=0,
                       for_status=1,
                       update_always=1,
                       props=None)

    vdef = wf.variables['time']
    vdef.setProperties(description="""Time of the last transition""",
                       default_value="""""",
                       default_expr="""state_change/getDateTime""",
                       for_catalog=0,
                       for_status=1,
                       update_always=1,
                       props=None)

    vdef = wf.variables['comments']
    vdef.setProperties(description="""Comments about the last transition""",
                       default_value="""""",
                       default_expr="""python:state_change.kwargs.get('comment', '')""",
                       for_catalog=0,
                       for_status=1,
                       update_always=1,
                       props=None)

    vdef = wf.variables['actor']
    vdef.setProperties(description="""The ID of the user who performed the last transition""",
                       default_value="""""",
                       default_expr="""user/getId""",
                       for_catalog=0,
                       for_status=1,
                       update_always=1,
                       props=None)

    vdef = wf.variables['review_history']
    vdef.setProperties(description="""Provides access to workflow history""",
                       default_value="""""",
                       default_expr="""state_change/getHistory""",
                       for_catalog=0,
                       for_status=0,
                       update_always=0,
                       props={'guard_permissions': 'Request review; Review portal content'})

    ## Worklists Initialization
    ldef = wf.worklists['reviewer_queue']
    ldef.setProperties(description="""Reviewer tasks""",
                       actbox_name="""Reviewer queue""",
                       actbox_url="""%(portal_url)s/search?review_state=pending""",
                       actbox_category="""global""",
                       props={'guard_permissions': 'PloneGossip: Review post; PloneGossip: Review thread', 'var_match_review_state': 'pending'})


def createPgthread_workflow(id):
    "..."
    ob = DCWorkflowDefinition(id)
    setupPgthread_workflow(ob)
    return ob

addWorkflowFactory(createPgthread_workflow,
                   id='pgthread_workflow',
                   title='PGThread Workflow [PloneGossip]')
